package consume

import (
	"applet/app/db"
	"applet/app/db/model"
	"applet/app/utils"
	"applet/app/utils/logx"
	"applet/consume/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/cc14514/go-geoip2"
	geoip2db "github.com/cc14514/go-geoip2-db"
	"github.com/streadway/amqp"
	"strings"
	"time"
)

func CanalFinWithdrawApplyForStatistics(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(500)
	delivery := ch.Consume(queue.Name, true) //设置自动应答

	geoIp2db, _ := geoip2db.NewGeoipDbByStatik()
	defer geoIp2db.Close()

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			//fmt.Println(string(res.Body))
			fmt.Println(">>>>>>>>>>>>>>>>>>CanalGuideOrderConsume<<<<<<<<<<<<<<<<<<<<<<<<<")
			err = CanalFinWithdrawApplyForStatisticsDetail(res.Body, geoIp2db)
			//_ = res.Reject(false)
			//_ = res.Ack(true)
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func CanalFinWithdrawApplyForStatisticsDetail(msg []byte, geoIp2db *geoip2.DBReader) error {
	//1、解析canal采集至mq中queue的数据结构体
	var canalMsg *md.CanalFinWithdrawApplyMessage[md.CanalFinWithdrawApply]
	err := json.Unmarshal(msg, &canalMsg)
	if err != nil {
		return err
	}

	//2、判断操作(目前只针对insert进行修改)
	if canalMsg.Type == md.CanalMsgUpdateSqlType {
		//4、构造数据结构体 && 插入es文档
		masterId := strings.Split(canalMsg.Database, "_")[1]
		sess := db.DBs[masterId].NewSession()
		defer sess.Close()
		sess.Begin()
		for _, data := range canalMsg.Data {
			if utils.InArr(data.State, []string{"2", "3"}) {
				flow := db.GetFinUserFlow(sess, data.Id)
				fee := "0"
				if flow != nil {
					fee = flow.SysFee
				}
				WithdrawAmount := data.Amount
				RealWithdrawAmount := data.Amount
				if data.FeeType == "1" {
					RealWithdrawAmount = utils.Float64ToStrByPrec(utils.StrToFloat64(RealWithdrawAmount)-utils.StrToFloat64(fee), 6)
				}
				month := utils.TimeParseStd(data.CreateAt).Format("200601")
				total := db.GetWithdrawMonthTotal(sess, month)
				if total == nil {
					total = &model.WithdrawMonthTotal{
						Month:          utils.StrToInt(month),
						WithdrawAmount: "",
						CreateAt:       utils.TimeParseStd(data.CreateAt),
						UpdateAt:       time.Now(),
					}
					_, err = sess.Insert(total)
					if err != nil {
						sess.Rollback()
						return err
					}
				}
				total.Fee = utils.Float64ToStrByPrec(utils.StrToFloat64(total.Fee)+utils.StrToFloat64(fee), 6)
				total.WithdrawAmount = utils.Float64ToStrByPrec(utils.StrToFloat64(total.WithdrawAmount)+utils.StrToFloat64(WithdrawAmount), 6)
				total.RealWithdrawAmount = utils.Float64ToStrByPrec(utils.StrToFloat64(total.RealWithdrawAmount)+utils.StrToFloat64(RealWithdrawAmount), 6)
				_, err = sess.Where("id=?", total.Id).Update(total)
				if err != nil {
					sess.Rollback()
					return err
				}
			}
		}
		sess.Commit()
	}
	return nil
}
