package consume

import (
	"applet/app/db"
	"applet/app/db/model"
	"applet/app/utils"
	"applet/app/utils/logx"
	"applet/consume/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/cc14514/go-geoip2"
	geoip2db "github.com/cc14514/go-geoip2-db"
	"github.com/streadway/amqp"
	"strings"
	"time"
	"xorm.io/xorm"
)

func CanalGuideOrderForStatistics(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(500)
	delivery := ch.Consume(queue.Name, true) //设置自动应答

	geoIp2db, _ := geoip2db.NewGeoipDbByStatik()
	defer geoIp2db.Close()

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			//fmt.Println(string(res.Body))
			fmt.Println(">>>>>>>>>>>>>>>>>>CanalGuideOrderConsume<<<<<<<<<<<<<<<<<<<<<<<<<")
			err = CanalGuideOrderForStatisticsDetail(res.Body, geoIp2db)
			//_ = res.Reject(false)
			//_ = res.Ack(true)
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func CanalGuideOrderForStatisticsDetail(msg []byte, geoIp2db *geoip2.DBReader) error {
	//1、解析canal采集至mq中queue的数据结构体
	var canalMsg *md.CanalOrderMessage[md.CanalGuideOrder]
	err := json.Unmarshal(msg, &canalMsg)
	if err != nil {
		return err
	}

	//2、判断操作(目前只针对update进行修改)
	if canalMsg.Type == md.CanalMsgUpdateSqlType {
		//4、构造数据结构体 && 插入es文档
		masterId := strings.Split(canalMsg.Database, "_")[1]
		sess := db.DBs[masterId].NewSession()
		defer sess.Close()
		sess.Begin()
		for _, data := range canalMsg.Data {

			if utils.StrToInt(data.PvdSettleAt) > 0 && strings.Contains(data.Ext, "pvd_settle_at") == false {
				err = comm(sess, data.OrdId, data.BenefitAll, "pvd_settle", data.PvdSettleAt, data.Pvd)
				if err != nil {
					sess.Rollback()
					return err
				}
			}
			if utils.StrToInt(data.SettleAt) > 0 && strings.Contains(data.Ext, "settle_at") == false {
				err = comm(sess, data.OrdId, data.BenefitAll, "settle", data.SettleAt, data.Pvd)
				if err != nil {
					sess.Rollback()
					return err
				}
			}
		}
		sess.Commit()
	}
	return nil
}
func comm(sess *xorm.Session, oid, commission, types, times, pvd string) error {
	byOid, _ := db.OrderListByOid(sess, oid)
	if byOid == nil {
		return nil
	}
	if strings.Contains(byOid.Ext, "settle_at") && types == "settle" {
		return nil
	}
	if strings.Contains(byOid.Ext, "pvd_settle_at") && types == "pvd_settle" {
		return nil
	}
	extData := make(map[string]interface{})
	json.Unmarshal([]byte(byOid.Ext), &extData)
	extData[types] = types
	byOid.Ext = utils.SerializeStr(extData)
	_, err2 := sess.Where("ord_id=?", byOid.OrdId).Cols("ext").Update(byOid)
	if err2 != nil {
		return err2
	}
	month := time.Unix(utils.StrToInt64(times), 0).Format("200601")
	total := db.GetGuideOrderMonthTotal(sess, month, pvd)
	if total == nil {
		total = &model.GuideOrderMonthTotal{
			Month:     utils.StrToInt(month),
			UnionType: pvd,
			CreateAt:  time.Unix(utils.StrToInt64(times), 0),
			UpdateAt:  time.Now(),
		}
		_, err := sess.Insert(total)
		if err != nil {
			return err
		}
	}
	if types == "settle" {
		sql := `select sum(amount) as amount from ord_list_relate where oid=%s `
		sql = fmt.Sprintf(sql, oid)
		data, _ := db.QueryNativeStringWithSession(sess, sql)
		if len(data) > 0 {
			total.StationSettleAmount = utils.Float64ToStrByPrec(utils.StrToFloat64(total.StationSettleAmount)+utils.StrToFloat64(data[0]["amount"]), 6)
		}
	}
	if types == "pvd_settle" {
		total.UnionSettleAmount = utils.Float64ToStrByPrec(utils.StrToFloat64(total.UnionSettleAmount)+utils.StrToFloat64(commission), 6)
	}
	_, err := sess.Where("id=?", total.Id).Update(total)
	if err != nil {
		return err
	}
	return nil
}
